@@ -226,6 +226,20 @@ varbuf_add_archqual(struct varbuf *vb, const struct dpkg_arch *arch)
 	varbuf_add_str(vb, arch->name);
 }
 
+/**
+ * Return a descriptive architecture name.
+ */
+const char *
+dpkg_arch_describe(const struct dpkg_arch *arch)
+{
+	if (arch->type == arch_none)
+		return C_("architecture", "<none>");
+	if (arch->type == arch_empty)
+		return C_("architecture", "<empty>");
+
+	return arch->name;
+}
+
 /**
  * Add a new foreign dpkg_arch architecture.
  */
@@ -275,16 +289,15 @@ dpkg_arch_load_list(void)
 
 	archfile = dpkg_db_get_path(DPKG_DB_ARCH_FILE);
 	fp = fopen(archfile, "r");
+	free(archfile);
 	if (fp == NULL) {
 		arch_list_dirty = true;
-		free(archfile);
 		return;
 	}
 
 	while (fgets_checked(archname, sizeof(archname), fp, archfile) >= 0)
 		dpkg_arch_add(archname);
 
-	free(archfile);
 	fclose(fp);
 }
 
