@@ -77,9 +77,6 @@ command_destroy(struct command *cmd)
 static void
 command_grow_argv(struct command *cmd, int need)
 {
-	/* We need a ghost byte for the NUL character. */
-	need++;
-
 	/* Check if we already have enough room. */
 	if ((cmd->argv_size - cmd->argc) >= need)
 		return;
@@ -216,17 +213,15 @@ command_shell(const char *cmd, const char *name)
 	const char *shell;
 	const char *mode;
 
-	if (cmd == NULL) {
-		mode = "-i";
-		shell = getenv("SHELL");
-	} else {
-		mode = "-c";
-		shell = NULL;
-	}
-
+	shell = getenv("SHELL");
 	if (str_is_unset(shell))
 		shell = DEFAULTSHELL;
 
+	if (cmd == NULL)
+		mode = "-i";
+	else
+		mode = "-c";
+
 	execlp(shell, shell, mode, cmd, NULL);
 	ohshite(_("unable to execute %s (%s)"), name, cmd);
 }
