@@ -1,5 +1,5 @@
 # Copyright © 2007-2009 Raphaël Hertzog <hertzog@debian.org>
-# Copyright © 2008-2009,2012-2014 Guillem Jover <guillem@debian.org>
+# Copyright © 2012 Guillem Jover <guillem@debian.org>
 #
 # This program is free software; you may redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -49,15 +49,15 @@ All the deps_* functions are exported by default.
 use strict;
 use warnings;
 
-our $VERSION = "1.01";
+our $VERSION = '1.02';
 
 use Dpkg::Version;
 use Dpkg::Arch qw(get_host_arch get_build_arch);
 use Dpkg::ErrorHandling;
 use Dpkg::Gettext;
 
-use base qw(Exporter);
-our @EXPORT = qw(deps_parse deps_eval_implication deps_compare);
+use Exporter qw(import);
+our @EXPORT = qw(deps_concat deps_parse deps_eval_implication deps_compare);
 
 # Some factorized function
 
@@ -134,17 +134,17 @@ sub _arch_is_superset {
 
 sub _arch_qualifier_allows_implication {
     my ($p, $q) = @_;
-    if (defined $p and $p eq "any") {
-	return 1 if defined $q and $q eq "any";
+    if (defined $p and $p eq 'any') {
+	return 1 if defined $q and $q eq 'any';
 	return 0;
-    } elsif (defined $p and $p eq "native") {
-	return 1 if defined $q and ($q eq "any" or $q eq "native");
+    } elsif (defined $p and $p eq 'native') {
+	return 1 if defined $q and ($q eq 'any' or $q eq 'native');
 	return 0;
     } elsif (defined $p) {
-	return 1 if defined $q and ($p eq $q or $q eq "any");
+	return 1 if defined $q and ($p eq $q or $q eq 'any');
 	return 0;
     } else {
-	return 0 if defined $q and $q ne "any" and $q ne "native";
+	return 0 if defined $q and $q ne 'any' and $q ne 'native';
 	return 1;
     }
 }
@@ -167,8 +167,8 @@ sub deps_eval_implication {
     my ($rel_p, $v_p, $rel_q, $v_q) = @_;
 
     # If versions are not valid, we can't decide of any implication
-    return undef unless defined($v_p) and $v_p->is_valid();
-    return undef unless defined($v_q) and $v_q->is_valid();
+    return unless defined($v_p) and $v_p->is_valid();
+    return unless defined($v_q) and $v_q->is_valid();
 
     # q wants an exact version, so p must provide that exact version.  p
     # disproves q if q's version is outside the range enforced by p.
@@ -239,7 +239,20 @@ sub deps_eval_implication {
         }
     }
 
-    return undef;
+    return;
+}
+
+=item my $dep = deps_concat(@dep_list)
+
+This function concatenates multiple dependency lines into a single line,
+joining them with ", " if appropriate, and always returning a valid string.
+
+=cut
+
+sub deps_concat {
+    my (@dep_list) = @_;
+
+    return join(', ', grep { defined $_ } @dep_list);
 }
 
 =item my $dep = deps_parse($line, %options)
@@ -260,6 +273,11 @@ Set to 0 to completely ignore that information.
 Define the host architecture. By default it uses
 Dpkg::Arch::get_host_arch() to identify the proper architecture.
 
+=item build_arch (defaults to the current architecture)
+
+Define the build architecture. By default it uses
+Dpkg::Arch::get_build_arch() to identify the proper architecture.
+
 =item reduce_arch (defaults to 0)
 
 If set to 1, ignore dependencies that do not concern the current host
@@ -287,6 +305,7 @@ sub deps_parse {
     $options{use_arch} = 1 if not exists $options{use_arch};
     $options{reduce_arch} = 0 if not exists $options{reduce_arch};
     $options{host_arch} = get_host_arch() if not exists $options{host_arch};
+    $options{build_arch} = get_build_arch() if not exists $options{build_arch};
     $options{union} = 0 if not exists $options{union};
     $options{build_dep} = 0 if not exists $options{build_dep};
 
@@ -300,11 +319,13 @@ sub deps_parse {
         foreach my $dep_or (split(/\s*\|\s*/m, $dep_and)) {
 	    my $dep_simple = Dpkg::Deps::Simple->new($dep_or, host_arch =>
 	                                             $options{host_arch},
+	                                             build_arch =>
+	                                             $options{build_arch},
 	                                             build_dep =>
 	                                             $options{build_dep});
 	    if (not defined $dep_simple->{package}) {
 		warning(_g("can't parse dependency %s"), $dep_or);
-		return undef;
+		return;
 	    }
 	    $dep_simple->{arches} = undef if not $options{use_arch};
             if ($options{reduce_arch}) {
@@ -329,9 +350,9 @@ sub deps_parse {
 	$dep_and = Dpkg::Deps::AND->new();
     }
     foreach my $dep (@dep_list) {
-        if ($options{union} and not $dep->isa("Dpkg::Deps::Simple")) {
-            warning(_g("an union dependency can only contain simple dependencies"));
-            return undef;
+        if ($options{union} and not $dep->isa('Dpkg::Deps::Simple')) {
+            warning(_g('an union dependency can only contain simple dependencies'));
+            return;
         }
         $dep_and->add($dep);
     }
@@ -348,7 +369,7 @@ This function is mainly used to implement the sort() method.
 =cut
 
 our %relation_ordering = (
-	'undef' => 0,
+	undef => 0,
 	REL_GE() => 1,
 	REL_GT() => 2,
 	REL_EQ() => 3,
@@ -370,13 +391,11 @@ sub deps_compare {
 	my @deps = $b->get_deps();
 	$b = $deps[0];
     }
-    my $ar = defined($a->{relation}) ? $a->{relation} : "undef";
-    my $br = defined($b->{relation}) ? $b->{relation} : "undef";
-    my $av = $a->{version} // '';
-    my $bv = $a->{version} // '';
+    my $ar = defined($a->{relation}) ? $a->{relation} : 'undef';
+    my $br = defined($b->{relation}) ? $b->{relation} : 'undef';
     return (($a->{package} cmp $b->{package}) ||
 	    ($relation_ordering{$ar} <=> $relation_ordering{$br}) ||
-	    ($av cmp $bv));
+	    ($a->{version} cmp $b->{version}));
 }
 
 
@@ -518,7 +537,7 @@ In the dependency "python:any (>= 2.6)", the arch qualifier is "any".
 
 =over 4
 
-=item $simple_dep->parse_string("dpkg-dev (>= 1.14.8) [!hurd-i386]")
+=item $simple_dep->parse_string('dpkg-dev (>= 1.14.8) [!hurd-i386]')
 
 Parses the dependency and modifies internal properties to match the parsed
 dependency.
@@ -540,7 +559,7 @@ use Dpkg::Version;
 use Dpkg::ErrorHandling;
 use Dpkg::Gettext;
 
-use base qw(Dpkg::Interface::Storable);
+use parent qw(Dpkg::Interface::Storable);
 
 sub new {
     my ($this, $arg, %opts) = @_;
@@ -557,11 +576,11 @@ sub new {
 
 sub reset {
     my ($self) = @_;
-    $self->{'package'} = undef;
-    $self->{'relation'} = undef;
-    $self->{'version'} = undef;
-    $self->{'arches'} = undef;
-    $self->{'archqual'} = undef;
+    $self->{package} = undef;
+    $self->{relation} = undef;
+    $self->{version} = undef;
+    $self->{arches} = undef;
+    $self->{archqual} = undef;
 }
 
 sub parse {
@@ -574,7 +593,7 @@ sub parse {
 sub parse_string {
     my ($self, $dep) = @_;
     return if not $dep =~
-            /^\s*                           # skip leading whitespace
+           m{^\s*                           # skip leading whitespace
               ([a-zA-Z0-9][a-zA-Z0-9+.-]*)  # package name
               (?:                           # start of optional part
                 :                           # colon for architecture
@@ -592,9 +611,9 @@ sub parse_string {
                 \s* \]                      # closing bracket
               )?                            # end of optional architecture
 	      \s*$			    # trailing spaces at end
-            /x;
+            }x;
     if (defined($2)) {
-	return if $2 eq "native" and not $self->{build_dep};
+	return if $2 eq 'native' and not $self->{build_dep};
 	$self->{archqual} = $2;
     }
     $self->{package} = $1;
@@ -611,13 +630,13 @@ sub output {
     my ($self, $fh) = @_;
     my $res = $self->{package};
     if (defined($self->{archqual})) {
-	$res .= ":" . $self->{archqual};
+	$res .= ':' . $self->{archqual};
     }
     if (defined($self->{relation})) {
-	$res .= " (" . $self->{relation} . " " . $self->{version} .  ")";
+	$res .= ' (' . $self->{relation} . ' ' . $self->{version} .  ')';
     }
-    if (defined($self->{'arches'})) {
-	$res .= " [" . join(" ", @{$self->{arches}}) . "]";
+    if (defined($self->{arches})) {
+	$res .= ' [' . join(' ', @{$self->{arches}}) . ']';
     }
     if (defined($fh)) {
 	print $fh $res;
@@ -632,14 +651,14 @@ sub implies {
     my ($self, $o) = @_;
     if ($o->isa('Dpkg::Deps::Simple')) {
 	# An implication is only possible on the same package
-	return undef if $self->{package} ne $o->{package};
+	return if $self->{package} ne $o->{package};
 
 	# Our architecture set must be a superset of the architectures for
 	# o, otherwise we can't conclude anything.
-	return undef unless Dpkg::Deps::_arch_is_superset($self->{arches}, $o->{arches});
+	return unless Dpkg::Deps::_arch_is_superset($self->{arches}, $o->{arches});
 
 	# The arch qualifier must not forbid an implication
-	return undef unless
+	return unless
 	    Dpkg::Deps::_arch_qualifier_allows_implication($self->{archqual},
 	                                                   $o->{archqual});
 
@@ -647,7 +666,7 @@ sub implies {
 	return 1 if not defined $o->{relation};
 	# If o has a version clause, we must also have one, otherwise there
 	# can't be an implication
-	return undef if not defined $self->{relation};
+	return if not defined $self->{relation};
 
 	return Dpkg::Deps::deps_eval_implication($self->{relation},
 		$self->{version}, $o->{relation}, $o->{version});
@@ -750,7 +769,7 @@ sub has_arch_restriction {
 
 sub get_evaluation {
     my ($self, $facts) = @_;
-    return undef if not defined $self->{package};
+    return if not defined $self->{package};
     return $facts->_evaluate_simple_dep($self);
 }
 
@@ -823,19 +842,19 @@ use warnings;
 
 use Dpkg::ErrorHandling;
 
-use base qw(Dpkg::Interface::Storable);
+use parent qw(Dpkg::Interface::Storable);
 
 sub new {
     my $this = shift;
     my $class = ref($this) || $this;
-    my $self = { 'list' => [ @_ ] };
+    my $self = { list => [ @_ ] };
     bless $self, $class;
     return $self;
 }
 
 sub reset {
     my ($self) = @_;
-    $self->{'list'} = [];
+    $self->{list} = [];
 }
 
 sub add {
@@ -890,7 +909,7 @@ sub is_empty {
 }
 
 sub merge_union {
-    internerr("The method merge_union() is only valid for Dpkg::Deps::Simple");
+    internerr('The method merge_union() is only valid for Dpkg::Deps::Simple');
 }
 
 package Dpkg::Deps::AND;
@@ -913,11 +932,11 @@ The output method uses ", " to join the list of sub-dependencies.
 use strict;
 use warnings;
 
-use base qw(Dpkg::Deps::Multiple);
+use parent -norequire, qw(Dpkg::Deps::Multiple);
 
 sub output {
     my ($self, $fh) = @_;
-    my $res = join(", ", map { $_->output() } grep { not $_->is_empty() } $self->get_deps());
+    my $res = join(', ', map { $_->output() } grep { not $_->is_empty() } $self->get_deps());
     if (defined($fh)) {
 	print $fh $res;
     }
@@ -945,7 +964,7 @@ sub implies {
 	}
 	return 1 if $subset;
     }
-    return undef;
+    return;
 }
 
 sub get_evaluation {
@@ -1018,11 +1037,11 @@ The output method uses " | " to join the list of sub-dependencies.
 use strict;
 use warnings;
 
-use base qw(Dpkg::Deps::Multiple);
+use parent -norequire, qw(Dpkg::Deps::Multiple);
 
 sub output {
     my ($self, $fh) = @_;
-    my $res = join(" | ", map { $_->output() } grep { not $_->is_empty() } $self->get_deps());
+    my $res = join(' | ', map { $_->output() } grep { not $_->is_empty() } $self->get_deps());
     if (defined($fh)) {
 	print $fh $res;
     }
@@ -1053,7 +1072,7 @@ sub implies {
 	}
 	return 1 if $subset;
     }
-    return undef;
+    return;
 }
 
 sub get_evaluation {
@@ -1126,11 +1145,11 @@ It uses $simple_dep->merge_union($other_dep) to get its job done.
 use strict;
 use warnings;
 
-use base qw(Dpkg::Deps::Multiple);
+use parent -norequire, qw(Dpkg::Deps::Multiple);
 
 sub output {
     my ($self, $fh) = @_;
-    my $res = join(", ", map { $_->output() } grep { not $_->is_empty() } $self->get_deps());
+    my $res = join(', ', map { $_->output() } grep { not $_->is_empty() } $self->get_deps());
     if (defined($fh)) {
 	print $fh $res;
     }
@@ -1139,12 +1158,12 @@ sub output {
 
 sub implies {
     # Implication test are not useful on Union
-    return undef;
+    return;
 }
 
 sub get_evaluation {
     # Evaluation are not useful on Union
-    return undef;
+    return;
 }
 
 sub simplify_deps {
@@ -1185,7 +1204,10 @@ use Dpkg::Version;
 sub new {
     my $this = shift;
     my $class = ref($this) || $this;
-    my $self = { 'pkg' => {}, 'virtualpkg' => {} };
+    my $self = {
+	pkg => {},
+	virtualpkg => {},
+    };
     bless $self, $class;
     return $self;
 }
@@ -1205,10 +1227,10 @@ Note that $multiarch is only used if $arch is provided.
 sub add_installed_package {
     my ($self, $pkg, $ver, $arch, $multiarch) = @_;
     my $p = {
-	"package" => $pkg,
-	"version" => $ver,
-	"architecture" => $arch,
-	"multi-arch" => $multiarch || "no",
+	package => $pkg,
+	version => $ver,
+	architecture => $arch,
+	multiarch => $multiarch || 'no',
     };
     $self->{pkg}{"$pkg:$arch"} = $p if defined $arch;
     push @{$self->{pkg}{$pkg}}, $p;
@@ -1261,28 +1283,28 @@ sub check_package {
 sub _find_package {
     my ($self, $dep, $lackinfos) = @_;
     my ($pkg, $archqual) = ($dep->{package}, $dep->{archqual});
-    return undef if not exists $self->{pkg}{$pkg};
+    return if not exists $self->{pkg}{$pkg};
     my $host_arch = $dep->{host_arch};
     my $build_arch = $dep->{build_arch};
     foreach my $p (@{$self->{pkg}{$pkg}}) {
-	my $a = $p->{"architecture"};
-	my $ma = $p->{"multi-arch"};
+	my $a = $p->{architecture};
+	my $ma = $p->{multiarch};
 	if (not defined $a) {
 	    $$lackinfos = 1;
 	    next;
 	}
 	if (not defined $archqual) {
-	    return $p if $ma eq "foreign";
-	    return $p if $a eq $host_arch or $a eq "all";
-	} elsif ($archqual eq "any") {
-	    return $p if $ma eq "allowed";
-	} elsif ($archqual eq "native") {
-	    return $p if $a eq $build_arch and $ma ne "foreign";
+	    return $p if $ma eq 'foreign';
+	    return $p if $a eq $host_arch or $a eq 'all';
+	} elsif ($archqual eq 'any') {
+	    return $p if $ma eq 'allowed';
+	} elsif ($archqual eq 'native') {
+	    return $p if $a eq $build_arch and $ma ne 'foreign';
 	} else {
 	    return $p if $a eq $archqual;
 	}
     }
-    return undef;
+    return;
 }
 
 sub _find_virtual_packages {
@@ -1313,12 +1335,20 @@ sub _evaluate_simple_dep {
 	next if defined $dep->{relation}; # Provides don't satisfy versioned deps
 	return 1;
     }
-    return undef if $lackinfos;
+    return if $lackinfos;
     return 0;
 }
 
 =head1 CHANGES
 
+=head2 Version 1.02
+
+=over
+
+=item * Add new Dpkg::deps_concat() function.
+
+=back
+
 =head2 Version 1.01
 
 =over
