@@ -304,7 +304,8 @@ ensure_packagefiles_available(struct pkginfo *pkg)
     while (thisline < loaded_list_end) {
       struct filenamenode *namenode;
 
-      if (!(ptr = memchr(thisline, '\n', loaded_list_end - thisline)))
+      ptr = memchr(thisline, '\n', loaded_list_end - thisline);
+      if (ptr == NULL)
         ohshit(_("files list file for package '%.250s' is missing final newline"),
                pkg_name(pkg, pnaw_nonambig));
       /* Where to start next time around. */
@@ -343,10 +344,8 @@ pkg_sorter_by_listfile_phys_offs(const void *a, const void *b)
    * INT_MAX. */
   if (pa->clientdata->listfile_phys_offs < pb->clientdata->listfile_phys_offs)
     return -1;
-  else if (pa->clientdata->listfile_phys_offs > pb->clientdata->listfile_phys_offs)
-    return 1;
   else
-    return 0;
+    return 1;
 }
 
 static void
@@ -652,6 +651,7 @@ struct filenamenode *findnamenode(const char *name, enum fnnflags flags) {
   newnode->next = NULL;
   newnode->divert = NULL;
   newnode->statoverride = NULL;
+  newnode->oldhash = NULL;
   newnode->newhash = EMPTYHASHFLAG;
   newnode->filestat = NULL;
   newnode->trig_interested = NULL;
@@ -660,6 +660,3 @@ struct filenamenode *findnamenode(const char *name, enum fnnflags flags) {
 
   return newnode;
 }
-
-/* vi: ts=8 sw=2
- */
