@@ -67,14 +67,20 @@ static int getwidth(void) {
   struct winsize ws;
   const char *columns;
 
-  if ((columns=getenv("COLUMNS")) && ((res=atoi(columns))>0))
-    return res;
-  else if (!isatty(1))
+  columns = getenv("COLUMNS");
+  if (columns) {
+    res = atoi(columns);
+    if (res > 0)
+      return res;
+  }
+
+  if (!isatty(1))
     return -1;
   else {
     res = 80;
 
-    if ((fd=open("/dev/tty",O_RDONLY))!=-1) {
+    fd = open("/dev/tty", O_RDONLY);
+    if (fd != -1) {
       if (ioctl(fd, TIOCGWINSZ, &ws) == 0)
         res = ws.ws_col;
       close(fd);
@@ -115,7 +121,7 @@ list_format_init(struct list_format *fmt, struct pkg_array *array)
       plen = strlen(pkg_name(array->pkgs[i], pnaw_nonambig));
       vlen = strlen(versiondescribe(&array->pkgs[i]->installed.version,
                                     vdew_nonambig));
-      alen = strlen(array->pkgs[i]->installed.arch->name);
+      alen = strlen(dpkg_arch_describe(array->pkgs[i]->installed.arch));
       pkg_summary(array->pkgs[i], &array->pkgs[i]->installed, &dlen);
 
       if (plen > fmt->nw)
@@ -221,7 +227,7 @@ list1package(struct pkginfo *pkg, struct list_format *fmt, struct pkg_array *arr
                     pkg_abbrev_eflag(pkg),
                     pkg_name(pkg, pnaw_nonambig),
                     versiondescribe(&pkg->installed.version, vdew_nonambig),
-                    pkg->installed.arch->name,
+                    dpkg_arch_describe(pkg->installed.arch),
                     pdesc, l);
 }
 
