@@ -173,7 +173,9 @@ badusage(char const *fmt, ...)
 	vfprintf(stderr, fmt, args);
 	va_end(args);
 	fprintf(stderr, "\n\n");
-	usage();
+	fprintf(stderr, _("Use '%s --help' for program usage information."),
+	        PROGNAME);
+	fprintf(stderr, "\n");
 	exit(2);
 }
 
@@ -401,29 +403,19 @@ log_msg(const char *fmt, ...)
 static int
 spawn(const char *prog, const char *args[])
 {
-	const char **cmd;
-	int i = 0;
 	pid_t pid, r;
 	int status;
 
-	while (args[i++]);
-	cmd = xmalloc(sizeof(char *) * (i + 2));
-	cmd[0] = prog;
-	for (i = 0; args[i]; i++)
-		cmd[i + 1] = args[i];
-	cmd[i + 1] = NULL;
-
 	pid = fork();
 	if (pid == -1)
 		error(_("fork failed"));
 	if (pid == 0) {
-		execvp(prog, (char *const *)cmd);
+		execvp(prog, (char *const *)args);
 		syserr(_("unable to execute %s (%s)"), prog, prog);
 	}
 	while ((r = waitpid(pid, &status, 0)) == -1 && errno == EINTR) ;
 	if (r != pid)
 		error(_("wait for subprocess %s failed"), prog);
-	free(cmd);
 
 	return status;
 }
@@ -443,8 +435,9 @@ subcall(const char *prog, ...)
 	va_end(args);
 
 	/* Prepare table for all parameters */
-	cmd = xmalloc(sizeof(*cmd) * (nb_opts + count + 1));
+	cmd = xmalloc(sizeof(*cmd) * (nb_opts + count + 2));
 	i = 0;
+	cmd[i++] = prog;
 	for (j = 0; j < nb_opts; j++)
 		cmd[i++] = pass_opts[j];
 	va_start(args, prog);
@@ -472,7 +465,7 @@ rename_mv(const char *src, const char *dst)
 		return false;
 
 	if (rename(src, dst) != 0) {
-		const char *args[3] = { src, dst, NULL };
+		const char *args[] = { "mv", src, dst, NULL };
 		int r;
 		r = spawn("mv", args);
 		if (WIFEXITED(r) && WEXITSTATUS(r) == 0)
@@ -659,6 +652,8 @@ struct commit_operation {
 struct alternative {
 	char *master_name;
 	char *master_link;
+	char *current;
+
 	enum alternative_status {
 		ALT_ST_UNKNOWN,
 		ALT_ST_AUTO,
@@ -672,6 +667,7 @@ struct alternative {
 
 	int ref_count;
 	bool modified;
+	bool known_current;
 };
 
 static void
@@ -698,11 +694,13 @@ alternative_new(const char *name)
 	alt = xmalloc(sizeof(*alt));
 	alt->master_name = xstrdup(name);
 	alt->master_link = NULL;
+	alt->current = NULL;
 	alt->status = ALT_ST_UNKNOWN;
 	alt->slaves = NULL;
 	alt->choices = NULL;
 	alt->commit_ops = NULL;
 	alt->modified = false;
+	alt->known_current = false;
 	alt->ref_count = 1;
 
 	return alt;
@@ -752,6 +750,8 @@ alternative_reset(struct alternative *alt)
 {
 	struct slave_link *slave;
 
+	free(alt->current);
+	alt->current = NULL;
 	free(alt->master_link);
 	alt->master_link = NULL;
 	while (alt->slaves) {
@@ -762,6 +762,7 @@ alternative_reset(struct alternative *alt)
 	alternative_choices_free(alt);
 	alternative_commit_operations_free(alt);
 	alt->modified = false;
+	alt->known_current = false;
 }
 
 static void
@@ -1039,8 +1040,8 @@ struct altdb_context {
 	char *filename;
 	enum altdb_flags flags;
 	bool modified;
-	void DPKG_ATTR_PRINTF(2) (*bad_format)(struct altdb_context *,
-	                                       const char *format, ...);
+	void DPKG_ATTR_NORET DPKG_ATTR_PRINTF(2)
+	     (*bad_format)(struct altdb_context *, const char *format, ...);
 	jmp_buf on_error;
 };
 
@@ -1197,12 +1198,10 @@ alternative_parse_fileset(struct alternative *a, struct altdb_context *ctx)
 		return false;
 	}
 
-	for (fs = a->choices; fs; fs = fs->next) {
-		if (strcmp(fs->master_file, master_file) == 0) {
-			free(master_file);
-			ctx->bad_format(ctx, _("duplicate path %s"),
-			                fs->master_file);
-		}
+	fs = alternative_get_fileset(a, master_file);
+	if (fs) {
+		free(master_file);
+		ctx->bad_format(ctx, _("duplicate path %s"), master_file);
 	}
 
 	if (stat(master_file, &st)) {
@@ -1238,8 +1237,6 @@ alternative_parse_fileset(struct alternative *a, struct altdb_context *ctx)
 			ctx->bad_format(ctx,
 			                _("priority of %s is out of range: %s"),
 			                master_file, prio_str);
-		free(prio_str);
-
 		fs = fileset_new(master_file, prio);
 		for (sl = a->slaves; sl; sl = sl->next) {
 			fileset_add_slave(fs, xstrdup(sl->name),
@@ -1247,8 +1244,6 @@ alternative_parse_fileset(struct alternative *a, struct altdb_context *ctx)
 		}
 		alternative_add_choice(a, fs);
 	}
-	free(master_file);
-
 	return true;
 }
 
@@ -1414,30 +1409,30 @@ alternative_save(struct alternative *a)
 	free(file);
 }
 
-static struct fileset *
-alternative_get_best(struct alternative *a)
+static const char *
+alternative_set_current(struct alternative *a, char *new_choice)
 {
-	struct fileset *fs, *best;
+	a->known_current = true;
+	a->current = new_choice;
 
-	for (best = fs = a->choices; fs; fs = fs->next)
-		if (fs->priority > best->priority)
-			best = fs;
-
-	return best;
+	return new_choice;
 }
 
-static char *
+static const char *
 alternative_get_current(struct alternative *a)
 {
 	struct stat st;
 	char *curlink;
 	char *file;
 
+	if (a->known_current)
+		return a->current;
+
 	xasprintf(&curlink, "%s/%s", altdir, a->master_name);
 	if (lstat(curlink, &st)) {
 		if (errno == ENOENT) {
 			free(curlink);
-			return NULL;
+			return alternative_set_current(a, NULL);
 		}
 		syserr(_("cannot stat file '%s'"), curlink);
 	}
@@ -1445,7 +1440,29 @@ alternative_get_current(struct alternative *a)
 	file = xreadlink(curlink);
 	free(curlink);
 
-	return file;
+	return alternative_set_current(a, file);
+}
+
+static struct fileset *
+alternative_get_best(struct alternative *a)
+{
+	struct fileset *fs, *best;
+	const char *current;
+
+	current = alternative_get_current(a);
+	if (current)
+		best = alternative_get_fileset(a, current);
+	else
+		best = NULL;
+
+	if (best == NULL)
+		best = a->choices;
+
+	for (fs = a->choices; fs; fs = fs->next)
+		if (fs->priority > best->priority)
+			best = fs;
+
+	return best;
 }
 
 static void
@@ -1453,7 +1470,7 @@ alternative_display_query(struct alternative *a)
 {
 	struct fileset *best, *fs;
 	struct slave_link *sl;
-	char *current;
+	const char *current;
 
 	pr("Name: %s", a->master_name);
 	pr("Link: %s", a->master_link);
@@ -1468,7 +1485,6 @@ alternative_display_query(struct alternative *a)
 		pr("Best: %s", best->master_file);
 	current = alternative_get_current(a);
 	pr("Value: %s", current ? current : "none");
-	free(current);
 
 	for (fs = a->choices; fs; fs = fs->next) {
 		printf("\n");
@@ -1488,7 +1504,7 @@ alternative_display_query(struct alternative *a)
 static void
 alternative_display_user(struct alternative *a)
 {
-	char *current;
+	const char *current;
 	struct fileset *fs;
 	struct slave_link *sl;
 
@@ -1497,7 +1513,6 @@ alternative_display_user(struct alternative *a)
 	current = alternative_get_current(a);
 	if (current) {
 		pr(_("  link currently points to %s"), current);
-		free(current);
 	} else {
 		pr(_("  link currently absent"));
 	}
@@ -1530,7 +1545,8 @@ alternative_display_list(struct alternative *a)
 static const char *
 alternative_select_choice(struct alternative *a)
 {
-	char *current, *ret, selection[_POSIX_PATH_MAX];
+	const char *current;
+	char *ret, selection[_POSIX_PATH_MAX];
 	struct fileset *best, *fs;
 	int len, idx;
 
@@ -1577,7 +1593,6 @@ alternative_select_choice(struct alternative *a)
 		         "or type selection number: "));
 		ret = fgets(selection, sizeof(selection), stdin);
 		if (ret == NULL || strlen(selection) == 0) {
-			free(current);
 			return NULL;
 		}
 		selection[strlen(selection) - 1] = '\0';
@@ -1591,7 +1606,6 @@ alternative_select_choice(struct alternative *a)
 			/* Look up by index */
 			if (idx == 0) {
 				alternative_set_status(a, ALT_ST_AUTO);
-				free(current);
 				return xstrdup(best->master_file);
 			}
 			idx--;
@@ -1599,17 +1613,14 @@ alternative_select_choice(struct alternative *a)
 				fs = fs->next;
 			if (fs) {
 				alternative_set_status(a, ALT_ST_MANUAL);
-				free(current);
 				return xstrdup(fs->master_file);
 			}
 		} else {
 			/* Look up by name */
-			for (fs = a->choices; fs; fs = fs->next) {
-				if (strcmp(fs->master_file, selection) == 0) {
-					alternative_set_status(a, ALT_ST_MANUAL);
-					free(current);
-					return xstrdup(selection);
-				}
+			fs = alternative_get_fileset(a, selection);
+			if (fs) {
+				alternative_set_status(a, ALT_ST_MANUAL);
+				return xstrdup(selection);
 			}
 		}
 	}
@@ -1830,7 +1841,8 @@ alternative_remove(struct alternative *a)
 static bool
 alternative_is_broken(struct alternative *a)
 {
-	char *altlnk, *wanted, *current;
+	const char *current;
+	char *altlnk, *wanted;
 	struct fileset *fs;
 	struct slave_link *sl;
 
@@ -1852,12 +1864,10 @@ alternative_is_broken(struct alternative *a)
 	if (current == NULL)
 		return true;
 
-	if (!alternative_has_choice(a, current)) {
-		free(current);
+	if (!alternative_has_choice(a, current))
 		return false;
-	}
+
 	fs = alternative_get_fileset(a, current);
-	free(current);
 
 	/* Check slaves */
 	for (sl = a->slaves; sl; sl = sl->next) {
@@ -2046,13 +2056,12 @@ alternative_get_selections(void)
 	alternative_map_load_names(alt_map_obj);
 
 	for (am = alt_map_obj; am && am->item; am = am->next) {
-		char *current;
+		const char *current;
 
 		current = alternative_get_current(am->item);
 		printf("%-30s %-8s %s\n", am->key,
 		       alternative_status_string(am->item->status),
 		       current ? current : "");
-		free(current);
 	}
 
 	alternative_map_free(alt_map_obj);
@@ -2256,7 +2265,7 @@ alternative_evolve(struct alternative *a, struct alternative *b,
 
 static void
 alternative_update(struct alternative *a,
-                   char *current_choice, const char *new_choice)
+                   const char *current_choice, const char *new_choice)
 {
 	/* No choice left, remove everything. */
 	if (!alternative_choices_count(a)) {
@@ -2440,7 +2449,8 @@ main(int argc, char **argv)
 	/* Set of files to install in the alternative. */
 	struct fileset *fileset = NULL;
 	/* Path of alternative we are offering. */
-	char *path = NULL, *current_choice = NULL;
+	char *path = NULL;
+	const char *current_choice = NULL;
 	const char *new_choice = NULL;
 	int i = 0;
 
@@ -2487,13 +2497,8 @@ main(int argc, char **argv)
 			prio = strtol(prio_str, &prio_end, 10);
 			if (prio_str == prio_end || *prio_end != '\0')
 				badusage(_("priority must be an integer"));
-			if (prio < INT_MIN || prio > INT_MAX || errno == ERANGE) {
-				/* XXX: Switch back to error on 1.17.x. */
-				prio = clamp(prio, INT_MIN, INT_MAX);
-				warning(_("priority is out of range: "
-				          "%s clamped to %ld"),
-				        prio_str, prio);
-			}
+			if (prio < INT_MIN || prio > INT_MAX || errno == ERANGE)
+				badusage(_("priority is out of range"));
 
 			a = alternative_new(argv[i + 2]);
 			inst_alt = alternative_new(argv[i + 2]);
